<?php

namespace Teknisa\Libs\Util;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\Messages;

class ReportDecorator {
    
    const BIRT_PATH_PARAMETER = 'birtReportPath'; 
    const QR2_PATH_PARAMETER  = 'qr2ReportPath';
    const NRORG_PARAMETER     = 'LOGIN_ORGANIZATION_ID';
    
    const BIRT_EXTENSION = 'RPTDESIGN';
    const QR2_EXTENSION  = 'QR2';
    
    protected $container;
    protected $nrorg;
    protected $windowPrefix;
    
    protected $qr2Path;
    protected $birtPath;
    
    public function __construct($environment) {
        $this->container = \Zeedhi\Framework\DependencyInjection\InstanceManager::getInstance()->getContainer();
        $this->nrorg = $environment->getNrorgTrab() != null ? $environment->getNrorgTrab(): $environment->getNrorg();

        if($this->nrorg === 1 || !$this->nrorg) {
            if ($this->container->hasParameter(self::NRORG_PARAMETER) && $this->container->getParameter(self::NRORG_PARAMETER) != null) {
                $this->nrorg = $this->container->getParameter(self::NRORG_PARAMETER);
            }
        }

        if ($this->container->hasParameter(self::BIRT_PATH_PARAMETER) && $this->container->getParameter(self::BIRT_PATH_PARAMETER) != null) {
            $this->birtPath = $this->container->getParameter(self::BIRT_PATH_PARAMETER);
        }
        if ($this->container->hasParameter(self::QR2_PATH_PARAMETER) && $this->container->getParameter(self::BIRT_PATH_PARAMETER) != null) {
            $this->qr2Path = $this->container->getParameter(self::QR2_PATH_PARAMETER);
        }
        
    }

    public function getData(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->windowPrefix = $row[0];
            
            $qr2Reports   = array();
            $birtReports    = array();
            $reportData     = array();
            
            $qr2Reports   = array_merge($this->getQr2Reports(), $qr2Reports);
            if(!empty($qr2Reports)) {
                $reportData = array_merge($reportData, $this->buildData($qr2Reports, $this->qr2Path,   self::QR2_EXTENSION));
            }
            $birtReports  = array_merge($this->getBirtReports(), $birtReports);
            if(!empty($birtReports)) {
                $reportData = array_merge($reportData, $this->buildData($birtReports, $this->birtPath, self::BIRT_EXTENSION));
            }

            $response->setDataSets(array(new DataSet('dataReport', $reportData)));
            
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }
    
    private function getQr2Reports() {
        return $this->getReportsMatched($this->qr2Path, self::QR2_EXTENSION, Messages::ERR_GET_QR2PATH_REPORTS_TO_BUILD_LIST);
    }
    
    private function getBirtReports() {
        return $this->getReportsMatched($this->birtPath, self::BIRT_EXTENSION, Messages::ERR_GET_BIRTPATH_REPORTS_TO_BUILD_LIST);
    }

    private function buildData($reportList, $baseDir, $extension) {
        
        $description = ($extension == self::QR2_EXTENSION) ?
            
            function($fileDir) {
                $fileString = utf8_encode(file_get_contents($fileDir));
                $descriptionStart = strpos($fileString, "þ") + 6;
                $descriptionEnd   = strpos($fileString, "TPF0") - 15;
                return substr($fileString, $descriptionStart, $descriptionEnd);
            }
            
            :
            
            function($fileDir) {
                $fileString = file_get_contents($fileDir);
                $title = '';
                $domResult = new \DOMDocument("1.0", "UTF-8");
                $domResult->loadXML($fileString);
                
                $properties = $domResult->getElementsByTagName("text-property");
                
                if(!empty($properties)) {
                    $propertie = $properties[0];
                    if(isset($propertie->attributes[0])) {
                        if($propertie->attributes[0]->value == 'title') {
                            $title = $propertie->nodeValue;
                        }
                    }
                }
                return $title;
            }
        ;
        
        $data = array();
        
        foreach($reportList as $report) {
            $fileDir = $baseDir.DIRECTORY_SEPARATOR.$report;
            
            $reportDescription = $description($fileDir);
            
     
            if($reportDescription) {
                
                $data[] = array(
                    'value' => $report,
                    'description' => $reportDescription
                ); 
            }
        }
        
        return $data;
    }

    private function getReportsMatched($dirToScan, $extension, $errorMess) {

        if (is_dir($dirToScan)) {
            $reportList = scandir($dirToScan);
    
            if ($reportList != false) {
                $dirNrorg = "$dirToScan".DIRECTORY_SEPARATOR."/$this->nrorg";
                
                
                $reports =  array_filter($reportList, function($report) use (&$extension) {
                    return $this->matchReport($report, $extension);
                });
                
                if (is_dir($dirNrorg)) {
                    $reportListNrorg = scandir($dirNrorg);

                    $reportsNrorg = array_filter($reportListNrorg, function($report) use (&$extension) {
                        return $this->matchReport($report, $extension);
                    });
                    
                    foreach($reportsNrorg as &$report) {
                        $report = $this->nrorg.DIRECTORY_SEPARATOR.$report;
                    }
                    
                    $reports = array_merge($reports, $reportsNrorg);
                    
                }
                
                return $reports;
            } else {
                throw new \Exception($errorMess, 1);
            }
        } else {
            throw new \Exception("$dirToScan não é um diretório.", 1);
        }
    }
    
    private function matchReport($report, $extension) {
        $regex = "/^(?i)$this->windowPrefix/";

        $resultMatch = preg_match($regex, $report);
        if ($resultMatch != 0) {
            return strpos(strtoupper($report), $extension) !== false;
        } else {
            return false;
        }
    }
    
}